@@ -14,9 +14,7 @@ def load_data(path='mnist.npz'):
     """
     path = get_file(path, origin='https://s3.amazonaws.com/img-datasets/mnist.npz')
     f = np.load(path)
-    x_train = f['x_train']
-    y_train = f['y_train']
-    x_test = f['x_test']
-    y_test = f['y_test']
+    x_train, y_train = f['x_train'], f['y_train']
+    x_test, y_test = f['x_test'], f['y_test']
     f.close()
     return (x_train, y_train), (x_test, y_test)
