@@ -1,13 +1,16 @@
 from __future__ import absolute_import
 from . import np_utils
-from . import conv_utils
-from . import data_utils
 from . import generic_utils
+from . import data_utils
 from . import io_utils
+from . import conv_utils
 
 # Globally-importable utils.
 from .io_utils import HDF5Matrix
 from .data_utils import get_file
+from .data_utils import Sequence
+from .data_utils import GeneratorEnqueuer
+from .data_utils import OrderedEnqueuer
 from .generic_utils import CustomObjectScope
 from .generic_utils import custom_object_scope
 from .generic_utils import get_custom_objects
